//----------------------------- BLK processing routines

function get_blk_by_path(func_name, blk, path, def_val)
{
  local arr=::split(path, "/");
  for (local i=0; i<arr.len()-1; ++i)
    blk=blk.getBlockByNameEx(arr[i]);

  return blk[func_name](arr[arr.len()-1], def_val);
}


function set_blk_by_path(func_name, blk, path, val)
{
  local arr=::split(path, "/");
  for (local i=0; i<arr.len()-1; ++i)
    blk=blk.addBlock(arr[i]);

  return blk[func_name](arr[arr.len()-1], val);
}


function get_blk_str (blk, path, def_val) { return ::get_blk_by_path("getStr" , blk, path, def_val); }
function get_blk_bool(blk, path, def_val) { return ::get_blk_by_path("getBool", blk, path, def_val); }
function get_blk_real(blk, path, def_val) { return ::get_blk_by_path("getReal" , blk, path, def_val); }
function get_blk_int (blk, path, def_val) { return ::get_blk_by_path("getInt" , blk, path, def_val); }

function set_blk_str (blk, path, val) { 
  if (typeof(val) == "string")
    return ::set_blk_by_path("setStr" , blk, path, val); 
}
function set_blk_bool(blk, path, val) { 
  if (typeof(val) == "bool")
    return ::set_blk_by_path("setBool", blk, path, val); 
}
function set_blk_real(blk, path, val) {
  if (typeof(val) == "float")
    return ::set_blk_by_path("setReal" , blk, path, val); 
}
function set_blk_int(blk, path, val) {
  if (typeof(val) == "integer")
    return ::set_blk_by_path("setInt" , blk, path, val); 
}

function set_values_to_gui(blk, scheme)
{
  foreach (name,desc in scheme)
  {
    local value=null;
    if ("getFromBlk" in desc)
      value=desc.getFromBlk(blk, desc);

    switch(desc.type)
    {
      case "string":
        if (value==null)
          value=::get_blk_str(blk, desc.blk, desc.defVal);
        setValue(name, value);
        break;

      case "bool":
        if (value==null)
          value=::get_blk_bool(blk, desc.blk, desc.defVal);
        setValue(name, value);
        break;

      case "real":
        if (value==null)
          value=::get_blk_real(blk, desc.blk, desc.defVal);
        setValue(name, value);
        break;

      case "int":
        if (value==null)
          value=::get_blk_int(blk, desc.blk, desc.defVal);
        setValue(name, value);
        break;
    }
  }
}


function get_values_from_gui(blk, scheme)
{
  foreach (name,desc in scheme)
  {
    switch(desc.type)
    {
      case "string":
        local value=getValue(name, desc.defVal);
        if ("setToBlk" in desc)
          desc.setToBlk(blk, desc, value);
        else
          ::set_blk_str(blk, desc.blk, value);
        break;

      case "bool":
        local value=getValue(name, desc.defVal);
        if ("setToBlk" in desc)
          desc.setToBlk(blk, desc, value);
        else
          ::set_blk_bool(blk, desc.blk, value);
        break;

      case "int":
        local value=getValue(name, desc.defVal);
        if ("setToBlk" in desc)
          desc.setToBlk(blk, desc, value);
        else
          ::set_blk_int(blk, desc.blk, value);
        break;

      case "real":
        local value=getValue(name, desc.defVal);
        if ("setToBlk" in desc)
          desc.setToBlk(blk, desc, value);
        else
          ::set_blk_real(blk, desc.blk, value);
        break;
    }
  }
}



class Settings
{
  scheme=null;
  fileNameLoad=null;
  fileNameSave=null;
  data=null;

  constructor(fn, schm)
  {
    scheme=schm;
    fileNameLoad = getSettingsBlkLoadPath(fn);
    fileNameSave = getSettingsBlkSavePath(fn);
    data=::DataBlock();
    data.load(fileNameLoad);
  }

  function updateGui()
  {
    ::set_values_to_gui(data, scheme);
  }

  function save()
  {
    data.load(fileNameLoad);
    ::get_values_from_gui(data, scheme);
    data.saveToTextFile(fileNameSave);
  }
}

